@@ -224,7 +224,9 @@ fn create_new_project(arg: NewArg) -> Result<()> {
 
 fn default_rule(id: &str, lang: SgLang) -> String {
   format!(
-    r#"id: {id}
+    r#"# yaml-language-server: $schema=https://raw.githubusercontent.com/ast-grep/ast-grep/main/schemas/rule.json
+
+id: {id}
 message: Add your rule message here....
 severity: error # error, warning, info, hint
 language: {lang}
