@@ -1,4 +1,5 @@
 mod deserialize_env;
+mod nth_child;
 pub mod referent_rule;
 mod relational_rule;
 mod stop_by;
@@ -8,6 +9,7 @@ pub use relational_rule::Relation;
 pub use stop_by::StopBy;
 
 use crate::maybe::Maybe;
+use nth_child::{NthChild, NthChildError, SerializableNthChild};
 use referent_rule::{ReferentRule, ReferentRuleError};
 use relational_rule::{Follows, Has, Inside, Precedes};
 
@@ -48,6 +50,11 @@ pub struct SerializableRule {
   /// A Rust regular expression to match the node's text. https://docs.rs/regex/latest/regex/#syntax
   #[serde(default, skip_serializing_if = "Maybe::is_absent")]
   pub regex: Maybe<String>,
+  /// `nth_child` accepts number, string or object.
+  /// It specifies the position in nodes' sibling list.
+  #[serde(default, skip_serializing_if = "Maybe::is_absent", rename = "nthChild")]
+  pub nth_child: Maybe<SerializableNthChild>,
+
   // relational
   /// `inside` accepts a relational rule object.
   /// the target node must appear inside of another node matching the `inside` sub-rule.
@@ -95,6 +102,7 @@ impl SerializableRule {
         pattern: self.pattern.into(),
         kind: self.kind.into(),
         regex: self.regex.into(),
+        nth_child: self.nth_child.into(),
       },
       relational: RelationalRule {
         inside: self.inside.into(),
@@ -112,11 +120,11 @@ impl SerializableRule {
   }
 }
 
-#[derive(Serialize, Deserialize, Clone, Default, JsonSchema)]
 pub struct AtomicRule {
   pub pattern: Option<PatternStyle>,
   pub kind: Option<String>,
   pub regex: Option<String>,
+  pub nth_child: Option<SerializableNthChild>,
 }
 
 /// A String pattern will match one single AST node according to pattern syntax.
@@ -133,7 +141,6 @@ pub enum PatternStyle {
   },
 }
 
-#[derive(Serialize, Deserialize, Clone, Default, JsonSchema)]
 pub struct RelationalRule {
   pub inside: Option<Box<Relation>>,
   pub has: Option<Box<Relation>>,
@@ -141,7 +148,6 @@ pub struct RelationalRule {
   pub follows: Option<Box<Relation>>,
 }
 
-#[derive(Serialize, Deserialize, Clone, Default)]
 pub struct CompositeRule {
   pub all: Option<Vec<SerializableRule>>,
   pub any: Option<Vec<SerializableRule>>,
@@ -154,6 +160,7 @@ pub enum Rule<L: Language> {
   Pattern(Pattern<L>),
   Kind(KindMatcher<L>),
   Regex(RegexMatcher<L>),
+  NthChild(NthChild<L>),
   // relational
   Inside(Box<Inside<L>>),
   Has(Box<Has<L>>),
@@ -168,7 +175,7 @@ pub enum Rule<L: Language> {
 impl<L: Language> Rule<L> {
   pub fn is_atomic(&self) -> bool {
     use Rule::*;
-    matches!(self, Pattern(_) | Kind(_) | Regex(_))
+    matches!(self, Pattern(_) | Kind(_) | Regex(_) | NthChild(_))
   }
   pub fn is_relational(&self) -> bool {
     use Rule::*;
@@ -199,6 +206,7 @@ impl<L: Language> Rule<L> {
       Rule::Pattern(p) => p.defined_vars(),
       Rule::Kind(_) => HashSet::new(),
       Rule::Regex(_) => HashSet::new(),
+      Rule::NthChild(n) => n.defined_vars(),
       Rule::Has(c) => c.defined_vars(),
       Rule::Inside(p) => p.defined_vars(),
       Rule::Precedes(f) => f.defined_vars(),
@@ -217,6 +225,7 @@ impl<L: Language> Rule<L> {
       Rule::Pattern(_) => Ok(()),
       Rule::Kind(_) => Ok(()),
       Rule::Regex(_) => Ok(()),
+      Rule::NthChild(n) => n.verify_util(),
       Rule::Has(c) => c.verify_util(),
       Rule::Inside(p) => p.verify_util(),
       Rule::Precedes(f) => f.verify_util(),
@@ -241,6 +250,7 @@ impl<L: Language> Matcher<L> for Rule<L> {
       Pattern(pattern) => pattern.match_node_with_env(node, env),
       Kind(kind) => kind.match_node_with_env(node, env),
       Regex(regex) => regex.match_node_with_env(node, env),
+      NthChild(nth_child) => nth_child.match_node_with_env(node, env),
       // relational
       Inside(parent) => match_and_add_label(&**parent, node, env),
       Has(child) => match_and_add_label(&**child, node, env),
@@ -261,6 +271,7 @@ impl<L: Language> Matcher<L> for Rule<L> {
       Pattern(pattern) => pattern.potential_kinds(),
       Kind(kind) => kind.potential_kinds(),
       Regex(regex) => regex.potential_kinds(),
+      NthChild(nth_child) => nth_child.potential_kinds(),
       // relational
       Inside(parent) => parent.potential_kinds(),
       Has(child) => child.potential_kinds(),
@@ -301,6 +312,8 @@ pub enum RuleSerializeError {
   InvalidKind(#[from] KindMatcherError),
   #[error("Rule contains invalid pattern matcher.")]
   InvalidPattern(#[from] PatternError),
+  #[error("Rule contains invalid nthChild.")]
+  NthChild(#[from] NthChildError),
   #[error("Rule contains invalid regex matcher.")]
   WrongRegex(#[from] RegexMatcherError),
   #[error("Rule contains invalid matches reference.")]
@@ -406,6 +419,9 @@ fn deserialze_atomic_rule<L: Language>(
   if let Some(regex) = atomic.regex {
     rules.push(R::Regex(RegexMatcher::try_new(&regex)?));
   }
+  if let Some(nth_child) = atomic.nth_child {
+    rules.push(R::NthChild(NthChild::try_new(nth_child, env)?));
+  }
   Ok(())
 }
 
